/*------------------------------------------------------------------------------
* Copyright (C) 2003-2006 Ben van Klinken and the CLucene Team
* 
* Distributable under the terms of either the Apache License (Version 2.0) or 
* the GNU Lesser General Public License, as specified in the COPYING file.
------------------------------------------------------------------------------*/
#include "CLucene/StdHeader.h"
#include "CJKAnalyzer.h"

#include "CLucene/util/VoidMap.h"
#include "CLucene/util/Reader.h"
#include "CLucene/analysis/AnalysisHeader.h"
#include "CLucene/analysis/Analyzers.h"
//#include "CLucene/analysis/standard/StandardFilter.h"
//#include "CLucene/analysis/standard/StandardTokenizer.h"
#include "CJKTokenizer.h"
#include "KoreanStemFilter.h"

CL_NS_USE(util)
CL_NS_USE(analysis)
//CL_NS_USE2(analysis, standard)
CL_NS_DEF2(analysis, cjk)
CL_NS_USE2(analysis, korean)

CJKAnalyzer::CJKAnalyzer():
    stopSet(false)
{
    StopFilter::fillStopTable( &stopSet,CL_NS(analysis)::StopAnalyzer::ENGLISH_STOP_WORDS);
}

CJKAnalyzer::CJKAnalyzer( const TCHAR** stopWords):
    stopSet(false)
{
    StopFilter::fillStopTable( &stopSet,stopWords );
}

CJKAnalyzer::~CJKAnalyzer(){
}


TokenStream* CJKAnalyzer::tokenStream(const TCHAR* fieldName, Reader* reader) 
{
    //TokenStream* ret = _CLNEW StandardTokenizer(reader);
    // StandardTokenizer에 CJK관련 코드가 있지만, 정상적으로 동작하지 않아
    // 그대로 복사해서 CJKTokenizer를 만들었음.
    // next(Token* t)함수에서 _CJK인지 먼저 체크하게만 바꿈.
    TokenStream* ret = _CLNEW CJKTokenizer(reader);
    ret = _CLNEW CL_NS2(analysis,standard)::StandardFilter(ret,true);
    ret = _CLNEW LowerCaseFilter(ret,true);
    ret = _CLNEW StopFilter(ret,true, &stopSet);
    ret = _CLNEW KoreanStemFilter(ret,true);
    return ret;
}

CL_NS_END2
