/*
 * Copyright (c) 2007 Chun-Koo Park
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <wchar.h>
#ifndef WIN32
#include <iconv.h>
#endif
#include <errno.h>
#include "CLucene.h"
#include "ConvertUtil.h"

#include <iostream>
#include <fstream>

#include <string>

using namespace std;

#ifndef WIN32

#define MAX_CONVERT_LENGTH (32768)

static iconv_t toUTF8 = NULL;
static iconv_t toUTF32 = NULL;

//static char *inoutCode= "EUC-KR";
static char *inoutCode= "UTF-8";
static char *internalCode= "UTF-32LE";


class init_iconv {
public:
    init_iconv();
    ~init_iconv();
} __init_iconv;

init_iconv::init_iconv()
{
    toUTF32 = iconv_open(internalCode, inoutCode);
    toUTF8 = iconv_open(inoutCode, internalCode);
}

init_iconv::~init_iconv()
{
    if (toUTF8) iconv_close(toUTF8);
    if (toUTF32) iconv_close(toUTF32);
}

int ConvertToUTF32(wchar_t* d,const char* s, size_t len)
{
    size_t dlen = len*sizeof(wchar_t);

    len = strlen(s);

    memset(d, 0, dlen);

    int cvtlen = iconv(toUTF32, (char **)&s, &len, (char **)&d, &dlen);

    if (cvtlen < 0) {
        printf("ConvertToUTF32 errno = %d (E2BIG=%d/EILSEQ=%d/EINVAL=%d)\n", errno, E2BIG, EILSEQ, EINVAL);
    }
    return cvtlen;
}


string ConvertToString(const wchar_t* _ws)
{
    wchar_t* ws = (wchar_t*)_ws;
    size_t wlen = wcslen(ws)*sizeof(wchar_t);
    char out[MAX_CONVERT_LENGTH];
    string ret;

    while (wlen > 0) {
        char *outp = out;
        size_t outlen = MAX_CONVERT_LENGTH;
        memset(out, 0, MAX_CONVERT_LENGTH);
        outlen--;

        int cvtlen = iconv(toUTF8, (char **)&ws, &wlen, &outp, &outlen);

        if (cvtlen < 0) {
            printf("ConvertToString errno = %d (E2BIG=%d/EILSEQ=%d/EINVAL=%d)\n", errno, E2BIG, EILSEQ, EINVAL);
            return ret;
        }
        ret += string(out);
    }

    return ret;
}

#else // ifdef WIN32

int ConvertToUTF32(TCHAR *tf, const char *f, int tf_len)
{
    unsigned int f_len = strlen(f);
    string ret;

    int ws_len = MultiByteToWideChar(CP_OEMCP, 0, f, f_len, (wchar_t*)tf, tf_len);
    if (ws_len > 0) {
        tf[ws_len] = 0;
    }
    return ws_len;
}

string ConvertToString(const wchar_t* _ws)
{
    string ret;
    int mul = 2;
    int ws_len = wcslen(_ws);
retry:
    int utf_len = ws_len * mul;
    char *utf_str = new char[utf_len+1];

    utf_len = WideCharToMultiByte(CP_OEMCP, 0, (LPCWSTR)_ws, ws_len, utf_str, utf_len, NULL, NULL);

    if (utf_len==0) {
        mul++;
        delete utf_str;
        if(mul > 6) {
            return "";
        }
        goto retry;
    }

    ret = string(utf_str, utf_len);
    delete utf_str;

    return ret;
}

#endif
